<?php
/**
 * Process the customer payment using the braintree PHP SDK. The $order_id is used to fetch the order amount to be charged.
 * @param int $order_id
 */
function process_customer_payment($order_id, Braintree_For_Woocommerce $gateway){
	global $woocommerce;
	
	$order = new WC_Order($order_id);
	if(!$order) {
		wc_add_notice(__('There has been an issue processing your order. Please try again', 'woocommerce'), 'error');
		return;
	}
	if(Braintree_For_Woocommerce::$is_woocommerce_subscription_active && wcs_order_contains_subscription($order)){
		return apply_filters('process_braintree_subscription', $order, $gateway);
	}
	
	return apply_filters('process_braintree_payment', $order, $gateway);
		
}
add_filter('process_customer_payment', 'process_customer_payment', 10, 2);
		




/**
 * Trigger the Braintree payment processing using the payment nonce. 
 * @param WC_Order $order_array
 */
function process_braintree_payment(WC_Order $order, Braintree_For_Woocommerce $gateway){
	if(Braintree_For_Woocommerce::$is_woocommerce_subscription_active && WC_Subscriptions_Change_Payment_Gateway::$is_request_to_change_payment) {
		return ['result'=>'success', 'redirect'=>Braintree_WooSubscription::get_paymentMethod_change_redirectUrl()];
	}
	
	$transaction_log = new BraintreeTransactionLog();
	
	$amount = $order->get_total();
	
	$nonce = $_REQUEST['payment_method_nonce'];
	$save_payment = isset($_POST['save_cc']) ? true : false;
	$paypal = 'PayPalAccount' == $_POST['payment_type'] ? true : false;
	$creditCard = 'CreditCard' == $_POST['payment_type'] ? true : false;
	$braintree_id = null;
	$token =  null;
	if($user_id = wp_get_current_user()->ID){
		$braintree_id = get_user_meta($user_id, 'braintree_vault_id', true);
	}

	
	$user_id = apply_filters('get_user_for_payment', $user_id);
	
	$args = array(
			'amount'=>$amount,
			'orderId'=>$gateway->order_prefix.$order->id,
			'options'=>array(
					'submitForSettlement'=>true
			),
			'customer'=>array(
					'firstName'=>isset($_POST['billing_first_name']) ? $_POST['billing_first_name'] : $order->billing_first_name,
					'lastName'=>isset($_POST['billing_last_name']) ? $_POST['billing_last_name'] : $order->billing_last_name,
			),
			'billing'=>array(
					'firstName'=>isset($_POST['billing_first_name']) ? $_POST['billing_first_name'] : $order->billing_first_name, 
					'lastName'=>isset($_POST['billing_last_name']) ? $_POST['billing_last_name'] : $order->billing_last_name,
					'streetAddress'=>isset($_POST['billing_address_1']) ? $_POST['billing_address_1'] : $order->billing_address_1,
					'postalCode'=>isset($_POST['billing_postcode']) ? $_POST['billing_postcode'] : $order->billing_postcode, 
					'countryCodeAlpha2'=>isset($_POST['billing_country']) ? $_POST['billing_country'] : $order->billing_country
			)
	);
	if($merchantAccountId = $gateway->getMerchantAccountId()){
		$args['merchantAccountId'] = $merchantAccountId;
	}
	
	if(!empty($_POST['use_saved_card'])){
		$token = $_POST['existing_card'];
	}
	if(!isset($_REQUEST['payment_method_nonce']) && $token == null){
		wc_add_notice(__('There has been an error verifying your payment', 'woocommerce'), 'error');
		return array('result'=>'failure', 'redirect'=>'');
	}
	
		if($token == null){
			$args['paymentMethodNonce'] = $nonce;
	  	
	}
	if($token != null){
		try {
			$customer = Braintree_Customer::find($braintree_id);
			$token = $customer->paymentMethods[$token]->token;
			$args['paymentMethodToken'] = $token;
			$save_payment = false;
			$args['customerId'] = $braintree_id;
			unset($args['paymentMethodNonce']);
		}
		catch(Exception $e) {
			wc_add_notice(__('The saved payment method could not be processed. Please try again.',
					 'Braintree_Woocommerce'), 'error');
			return array('result'=>'failure', 'redirect'=>'');
		}
	}
	
	$args = apply_filters('braintree_payment_arguments', $args);
	
	if($save_payment && $user_id){
		$args['options']['storeInVaultOnSuccess'] = true;
		
		if($braintree_id){
			if(has_braintree_id($braintree_id)){
				
				$result = Braintree_PaymentMethod::create(array(
						'customerId'=>$braintree_id,
						'paymentMethodNonce'=>$nonce,
						'billingAddress'=>array('firstName'=>$_POST['billing_first_name'], 
							'lastName'=>$_POST['billing_last_name'],
							'streetAddress'=>$_POST['billing_address_1'],
							'postalCode'=>$_POST['billing_postcode'], 
							'countryCodeAlpha2'=>$_POST['billing_country']
			)
				));
				if($result->success){
				$args['paymentMethodToken'] = $result->paymentMethod->token;
				unset($args['paymentMethodNonce']);
				}
			}
			else $braintree_id = null;
		}
			else {
				$args['paymentMethodNonce'] = $nonce;
			}
		}
	try {	
	$result = Braintree_Transaction::sale($args);
	}
	catch(Exception $e){
		wc_add_notice(__('Your payment could not be processed at this time', 'Braintree_Woocommerce'), 'error');
		$transaction_log->writeToLogFile($e);
		return ['result'=>'failure', 'redirect'=>''];
	}
	
	do_action('process_braintree_transaction_result', $result, $order);
	
	if($result->success){
		global $woocommerce;
		if($gateway->order_status == 'wc-completed'){
			$order->payment_complete($result->transaction->id);
		}
		else{
			$order->update_status($gateway->order_status, __('Credit card payment processed', 'woocommerce'));
		}
		$woocommerce->cart->empty_cart();
		update_post_meta($order->id, '_transaction_id', $result->transaction->id);
		if($result->transaction->paymentInstrumentType == 'credit_card'){
			$cardDetails = $result->transaction->creditCardDetails;
			update_post_meta($order->id, '_payment_method_title', $cardDetails->cardType.' - '.$cardDetails->maskedNumber);
		}
		elseif($result->transaction->paymentInstrumentType == 'paypal_account') {
			$paypalDetails = $result->transaction->paypalDetails;
			update_post_meta($order->id, '_payment_method_title', 'PayPal - '.$paypalDetails->payerEmail);
		}
			if($user_id && !$braintree_id){
				$new_id = $result->transaction->customerDetails->id;
				update_user_meta($user_id, 'braintree_vault_id', $new_id);
			}
		$transaction_log->writeToLogFile('Transaction: '.$result->transaction->id.' User ID: '.$user_id.' Order ID: '.$order->id.' Payment Method: '.$order->payment_method_title);	
		return array(
				'result'=>'success',
				'redirect'=>$gateway->get_return_url($order));
	}
	elseif(!$result->success && $result->transaction){
		$transaction_log->writeFailedTransactionToLog($result, 'Order');
		if($result->transaction->gatewayRejectionReason == 'avs') {
			wc_add_notice(__('Please check your billing postal code and address.', 'braintree_woocommerce'), 'error');
			return array(
					'result'=>'failure',
					'redirect'=>'');
		}
		wc_add_notice($result->message, 'error');
		return array(
				'result'=>'failure',
				'redirect'=>''
		);
	}
	else {
		if($result->errors->deepSize() > 0){
			foreach($result->errors->deepAll() as $error){
				wc_add_notice($error->message, 'error');
				$transaction_log->writeFailedTransactionToLog($error, 'Order');
			}
		}
		else {
			wc_add_notice($result->message, 'error');
		}
		return array(
				'result'=>'fail',
				'redirect'=>''
		);
	}
	
}
add_filter('process_braintree_payment', 'process_braintree_payment', 10, 2);


function get_credit_cardTypes(){
	$card_types = array('American Express', 'MasterCard', 'Discover', 'Visa', 'Diners Club', 'Carte', 'China UnionPay',  
			'JCB', 'Laser', 'Maestro');
	return $card_types;
}

function card_type_html($card_types){
	foreach($card_types as $card){
		$html .= '<option id="'.$card.'" class="cc_option">'.$card.'</option>';
	}
	return $html;
}

function get_expiration_months_html(){
	/* $months = array(
			'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September',
			'October', 'November', 'December'); */
	$months = array('01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12');
	
	//$count = 1;
	foreach($months as $month){
		$html .= '<option id="'.$month.'">'.$month.'</option>';
	}
	return $html;
}

function get_expiration_years(){
	$current_year = (int) date('Y');
	$count = 10;
	for($i = 0; $i < $count; $i++){
		$html .= '<option class="expire_year">'.$current_year.'</option>';
		$current_year++;
	}
	return $html;
}

function send_braintree_client_token($braintree_id = false){
	if($braintree_id){
		try{
			return Braintree_ClientToken::generate(array('customerId'=>$braintree_id));
		}
		catch(Exception $e){
			return send_braintree_client_token();
		}
	 }
	 try{
	 	$token = Braintree_ClientToken::generate();
	 }
	 catch(Exception $e){
	 	$client_token = '';
	 }
	return $token;
}

function get_braintree_checkout_ui(Braintree_For_Woocommerce $gateway){
	if($gateway->only_paypal){
		$gateway->get_paypalOnlyCheckout();
	}
	else if($gateway->form_type == 'custom'){
		get_custom_checkout_ui($gateway);
	}
	else {
		get_braintree_dropin_ui($gateway);
	}
}
add_filter('get_braintree_checkout_ui', 'get_braintree_checkout_ui', 10, 2);

function get_custom_checkout_ui(Braintree_For_Woocommerce $gateway){
	$wp_user = wp_get_current_user();
	if($wp_user){
		$braintree_id = get_user_meta($wp_user->ID, 'braintree_vault_id', true);
	}
	$user = get_current_braintree_user($braintree_id);
	$user = apply_filters('get_braintree_user', $user, $gateway);
	
	switch ($gateway->payment_form_name){
		case 'payment_form_0':
			return call_user_func('braintree_payment_form_0', array('gateway'=>$gateway, 'user'=>$user));
			break;
			
		case 'payment_form_1':
			return call_user_func('braintree_payment_form_1', array('gateway'=>$gateway, 'user'=>$user));
			break;	
			
		case 'payment_form_2':
			return call_user_func('braintree_payment_form_0', array('gateway'=>$gateway, 'user'=>$user));
			break;
			
	}
	
}

function get_braintree_dropin_ui($gateway){
	$wp_user = wp_get_current_user();
	$braintree_id = false;
	if($wp_user){
		$braintree_id = get_user_meta($wp_user->ID, 'braintree_vault_id', true);
	}
	/* $user = get_current_braintree_user($braintree_id);
	
	$user = apply_filters('get_braintree_user', $user, $gateway); */
	?>
	<div class="payment-information-container">
	<?php echo $gateway->get_active_environment()?>
	<?php if(Braintree_For_Woocommerce::$is_woocommerce_subscription_active && WC_Subscriptions_Change_Payment_Gateway::$is_request_to_change_payment) {
		?>
		<div><span><?php echo __('Billing Address', 'braintree_woocommerce')?></span></div>
		<span><input type="text" name="billing_address1"/></span>
		<div><span><?php echo __('Billing Country', 'braintree_woocommerce')?></span></div>
		<span><select class="paymentmethod-country" name="billing_country">
		<?php foreach(Braintree_Donation_Countries::$countries as $country => $text) {?>
						<option value="<?php echo $country?>"><?php echo $text?></option>
		<?php }?>
			  </select></span>
	</div>
	<?php }?>
	<div id="dropin-container"><div class="save-cc"><div><span>Save CC</span></div><input type="checkbox" name="save_cc" value="save"/></div>
		<?php echo $gateway->get_accepted_paymentMethods()?>
	</div>
	<input type="hidden" name="payment_type"/>
	<input type="hidden" id="payment_form" value="dropin"/>
	<input type="hidden" name="payment_method_nonce" id="payment_method_nonce"/>
	<input type="hidden" id="form_submitted" value="false"/>
	  <script>
	 	 if(typeof dropinCheckout === 'undefined'){
				var dropinCheckout = null;
	 	 }
			window.formInitialized = false;
			var BraintreeActions =  function(){};
			BraintreeActions.prototype.validateNonce = function(){
				if($('input[name="payment_method_nonce"]').val() !== ""){
					return true;
				}
				else {
					return false;
				}
			}
	     var braintreeActions = new BraintreeActions();
		 var $ = jQuery;
		 var clientToken = "<?php echo send_braintree_client_token($braintree_id)?>";

			<?php if(isset($_REQUEST['change_payment_method']) || 
					isset($_REQUEST['pay_for_order']) && $_REQUEST['pay_for_order'] === 'true'){?>
				jQuery(document).ready(function(){
					var $ = jQuery;
					braintree.setup(clientToken, 'dropin', {container: 'dropin-container',
						form: 'order_review',
						onPaymentMethodReceived: function (response){
							$("#payment_method_nonce").val(response.nonce);
							$("#order_review").submit();
	
							}});
				});
			
			<?php } else {?>
			 jQuery(document).ready(function(){
				 var $ = jQuery;
				 var checkoutFormInitialized = false;
				 var teardownCalled = false;
				form = $('form.checkout').attr('id', 'checkout');
				function initializeDropinForm(){
					if(typeof dropinCheckout === 'object' && dropinCheckout !== null){
						dropinCheckout.teardown(function(){
							dropinCheckout = null;
						});
					}
					else if (formInitialized === true){
						return;
					}
						braintree.setup(clientToken, 'dropin', {container: 'dropin-container',
							form: 'checkout',
							onPaymentMethodReceived: function (response){
							$("input[name='payment_method_nonce']").val(response.nonce);
							$('form.checkout').submit();
								}, 
								onReady: function(integration){
									dropinCheckout = integration;
								}
						});
						formInitialized = true;
						$('form.checkout').on('checkout_place_order', braintreeActions.validateNonce);
				}
			
				initializeDropinForm();
				$('input[name="payment_method"]').on('change', function(e){
					if($('#payment_method_braintree_payment_gateway').is(':checked')){
						if(dropinCheckout == null){
							initializeDropinForm();
						}
					}
					else {
						if(dropinCheckout != null){
							dropinCheckout.teardown(function(){
								dropinCheckout = null;
						});
							formInitialized = false;
						}
						$('form.checkout').off('checkout_place_order', braintreeActions.validateNonce);
					}
				});

			 });
			<?php }?>  
  </script>
  <style>
  
  div.save-cc span {
  	font-size: 14px;
  	font-weight: 700;
  }
  div.save-cc input[type="checkbox"] {
  	width: 20px;
  	height: 20px;
  }
  </style>
	<?php 
}

/**
 * This function replaces the standard woocommerce payment-method.php page with an exact replica. This replacement is needed
 * because there is a bug in either woocommerce or braintree that prevents the dropin ui from staying within the html container
 * on the checkout page. 
 * @param unknown $template_name
 * @param unknown $template_path
 * @param unknown $default_path
 */
/* function replace_woocommerce_payment_methods_page($located, $template_name, $args, $template_path, $default_path){
	if($template_name !== 'checkout/payment-method.php') {
		return $located;
	}
	$settings = get_option('woocommerce_braintree_payment_gateway_settings', true);
	if($settings['braintree_dropin_ui'] == 'yes' || $settings['braintree_custom_ui']){
	$located = WC_BRAINTREE_GATEWAY.'includes/templates/checkout/payment-method.php';
		return $located;
	}
	return $located;
}
add_filter('wc_get_template', replace_woocommerce_payment_methods_page, 10, 5); */

function get_hidden_cart_total(){
	$cart = WC()->cart;
	if($cart->prices_include_tax){
		return $cart->cart_contents_total;
	}
	else {
		return ($cart->cart_contents_total + $cart->tax_total);
	}
}
add_filter('get_hidden_cart_total', 'get_hidden_cart_total');

function get_braintree_template_scripts($located, $handle, $script){
	$template_path = get_template_directory();
	
	if(!$template_path){
		return $located;
	}
	if(file_exists($template_path.'/braintree-template/assets/'.$script)){
		return get_template_directory_uri().'/braintree-template/assets/';
	}
	return $located;
}
add_filter('get_braintree_template_scripts', 'get_braintree_template_scripts', 10, 3);

function has_braintree_id($user_id){
	try {
		if($customer = Braintree_Customer::find($user_id)){
			return true;
		}
	}
	catch(Exception $e){
		return false;
	}
}

function get_current_braintree_user($user_id){
	try{
	$user = Braintree_Customer::find($user_id);
		return $user;
	}
	catch(Exception $e){
		return false;
	}
}

function get_paypal_screen($gateway){
	
	if($gateway->paypal_enabled){
		?>
		<div class="paypal-div"><h3><?php echo __('Paypal', 'woocommerce')?></h3></div>
		  					<div id="paypal-container"></div>
		  					<input type="hidden" id="order_amount" value="<?php echo apply_filters('get_hidden_cart_total')?>"/>
		  					<script>
		  				
		  				jQuery(document).ready(function(){
							var paypalCheckout = null;
		  					var $ = jQuery;
		  					var amount = $('#order_amount').val();
		  					var clientToken = "<?php echo send_braintree_client_token(false)?>";			
							function initializePaypalCheckout(){
			  					braintree.setup(clientToken, 'paypal', {
			  					  	container: "paypal-container",
			  					  	onPaymentMethodReceived: function (response) {
			  					  	$("input[name=\'payment_method_nonce\']").val(response.nonce);
			  					  },
			  					  onReady: function(integration){
									paypalCheckout = integration;
			  					  }
			  					});
							}
							$(document.body).on('updated_checkout', function(){
								if(typeof paypalCheckout === 'object' && paypalCheckout !== null){
									paypalCheckout.teardown(function(){
										paypalCheckout = null;
									});
									initializePaypalCheckout();
								}
							});
						    
							initializePaypalCheckout();
		  				})
		  					</script>
		  					<?php }
	}
	
	function get_braintree_setup_js($type = 'custom'){
		$wp_user = wp_get_current_user();
		$braintree_id = false;
		if($wp_user){
			$braintree_id = get_user_meta($wp_user->ID, 'braintree_vault_id', true);
		}
		if($type == 'custom'){
			
		return '<script type="text/javascript">
 		var customCheckout = null;
 		jQuery(document).ready(function(){
		var BraintreeActions =  function(){};
			BraintreeActions.prototype.validateNonce = function(){
				if($(\'input[name="payment_method_nonce"]\').val() !== ""){
					return true;
				}
				else if(document.getElementById("use_saved_card").checked) {
					return true;
				}
				else{
					return false;
				}
			}
	     var braintreeActions = new BraintreeActions();
		 var $ = jQuery;
			var clientToken = "'.send_braintree_client_token($braintree_id).'";
			form = $("form[name=\'checkout\']").attr("id", "checkout");
			if($("#order_review").length && $("#order_review").is("form")){
				braintree.setup(clientToken, "custom", {id: "order_review", 
					onPaymentMethodReceived: function (response){
							$("input[name=\'payment_method_nonce\']").val(response.nonce);
 							$("#payment_type").val(response.type);
 							$("#order_review").submit();
						}, 
					onReady: function(integration){
							
 						}
					});
 			}
 		else {
 		function initializeCustomForm(){
 			braintree.setup(clientToken, "custom", {id: "checkout", 
					onPaymentMethodReceived: function (response){
 						
 						$("input[name=\'payment_method_nonce\']").val(response.nonce);
 						$("form.checkout").submit();
						},
					onReady: function(integration){
 						customCheckout = integration;
 					}
				});
				$("form.checkout").on("checkout_place_order", braintreeActions.validateNonce);
 		}
 			$(document.body).on("updated_checkout", function(e){
 					if(typeof customCheckout === "object" && customCheckout !== null){
 						customCheckout.teardown(function(){
							customCheckout = null;
						});
					initializeCustomForm();
 					}
 			});
			initializeCustomForm();
 				$("input[name=\'payment_method\']").on("change", function(e){
 					if($("#payment_method_braintree_payment_gateway").is(":checked")){
 						initializeCustomForm();
 					}
 					else {
						if(customCheckout != null){
							customCheckout.teardown(function(){
								customCheckout = null;
	 						});
	 					}
			$("form.checkout").off("checkout_place_order", braintreeActions.validateNonce);
 				}});
			
			
 		}

  })   
 		</script>';
		}
	}

?>